// Generated by CoffeeScript 2.3.1
// Export Plugin
module.exports = function(BasePlugin) {
  var PartialsPlugin, Task, TaskGroup, extendr, pathUtil, util;
  // Requires
  extendr = require('extendr');
  ({Task, TaskGroup} = require('taskgroup'));
  pathUtil = require('path');
  util = require('util');
  return PartialsPlugin = (function() {
    // Define Plugin
    class PartialsPlugin extends BasePlugin {
      
      // -----------------------------
      // Initialize

      // Prepare our Configuration
      setConfig(...args) {
        var config, docpadConfig;
        // Apply
        super.setConfig(...args);
        // Adjust
        docpadConfig = this.docpad.getConfig();
        config = this.getConfig();
        // ensure config name backward compatibility
        config.partialPaths = config.partialsPath || config.partialPaths;
        // ensure the partialPaths is an array
        if (!util.isArray(config.partialPaths)) {
          config.partialPaths = [config.partialPaths];
        }
        // Adjust
        config.partialPaths.forEach(function(partialPath, index) {
          config.partialPaths[index] = pathUtil.resolve(docpadConfig.srcPath, partialPath);
        });
        return this;
      }

      // -----------------------------
      // Events

      // Populate Collections
      populateCollections(opts, next) {
        var config, docpad, processedCount;
        // Prepare
        config = this.config;
        docpad = this.docpad;
        // Load our partials directory
        processedCount = 0;
        config.partialPaths.forEach(function(partialPath) {
          docpad.parseDocumentDirectory({
            path: partialPath
          }, function(err, results) {
            if (err || processedCount === (config.partialPaths.length - 1)) {
              next(err);
            }
            processedCount++;
          });
        });
        return this;
      }

      // Extend Collections
      extendCollections(opts) {
        var config, database, docpad, locale;
        // Prepare
        config = this.getConfig();
        docpad = this.docpad;
        locale = this.locale;
        database = docpad.getDatabase();
        // Add our partials collection
        docpad.setCollection(config.collectionName, database.createLiveChildCollection().setQuery('isPartial', {
          $or: {
            isPartial: true,
            fullPath: {
              $startsWith: config.partialPaths
            }
          }
        }).on('add', function(model) {
          docpad.log('debug', util.format(locale.addingPartial, model.getFilePath()));
          return model.setDefaults({
            isPartial: true,
            render: false,
            write: false
          });
        }));
        return this;
      }

      // -----------------------------
      // Rendering

      // Render Partial
      // Render a partial asynchronously
      // next(err,result,document)
      renderPartial(partial, next) {
        var cacheable, docpad, locale, partialsCache, ref, ref1, result;
        // Prepare
        docpad = this.docpad;
        locale = this.locale;
        partialsCache = this.partialsCache;
        result = null;
        // Check if our partial is cacheable
        cacheable = (ref = partial.document.getMeta().get('cacheable')) != null ? ref : false;
        if (cacheable === true) {
          result = (ref1 = partialsCache[partial.cacheId]) != null ? ref1 : null;
        }
        if (result != null) {
          // Got from cache, so use that
          return next(null, result);
        }
        // Render
        docpad.renderDocument(partial.document, {
          templateData: partial.data
        }, function(err, result, document) {
          if (err) {
            // Check
            return next(err);
          }
          // Cache
          if (cacheable === true) {
            partialsCache[partial.cacheId] = result;
          }
          // Forward
          return next(null, result);
        });
        return this;
      }

      // Extend Template Data
      // Inject our partial methods
      extendTemplateData({templateData}) {
        var docpad, locale, me;
        // Prepare
        me = this;
        docpad = this.docpad;
        locale = this.locale;
        // Apply
        templateData.partial = function(partialName, ...objs) {
          var collection, config, err, file, i, len, message, obj, partial, ref;
          // Reference others
          config = me.getConfig();
          if (typeof this.referencesOthers === "function") {
            this.referencesOthers();
          }
          // Prepare
          file = this.documentModel;
          partial = {};
          // Fetch our partial
          //partialFuzzyPath = pathUtil.join(config.partialsPath, partialName)
          //partial.document ?= docpad.getCollection('partials').fuzzyFindOne(partialFuzzyPath)
          collection = docpad.getCollection('partials');
          config.partialPaths.forEach(function(partialPath) {
            var partialFuzzyPath;
            partialFuzzyPath = pathUtil.join(partialPath, partialName);
            if (partial.document == null) {
              partial.document = collection.fuzzyFindOne(partialFuzzyPath);
            }
          });
          if (!partial.document) {
            // Partial was not found
            message = util.format(locale.partialNotFound, partialName);
            err = new Error(message);
            if (partial.err == null) {
              partial.err = err;
            }
            return message;
          }
          // Prepare the initial partial data
          partial.data = {};
          // If no object is provided then provide the current template data as the first thing
          // if the performance first option is set to false (the default)
          if (config.performanceFirst === false) {
            if ((ref = objs[0]) !== false && ref !== this) {
              objs.unshift(this);
            }
          }
// Cycle through the objects merging them together
// ignore boolean values
          for (i = 0, len = objs.length; i < len; i++) {
            obj = objs[i];
            if (obj && (obj !== true)) {
              extendr.extend(partial.data, obj);
            }
          }
          // ^ why do we just do a shallow extend here instead of a deep extend?

          // Prepare our partial id
          partial.path = partial.document.getFilePath();
          partial.cacheId = partial.document.id;
          partial.id = Math.random(); // require('crypto').createHash('md5').update(partial.cacheId+'|'+JSON.stringify(partial.data)).digest('hex')
          partial.container = '[partial:' + partial.id + ']';
          // Check if a partial with this id already exists!
          if (me.foundPartials[partial.id]) {
            return partial.container;
          }
          // Store the partial
          me.foundPartials[partial.id] = partial;
          // Create the task for our partial
          partial.task = new Task(`renderPartial: ${partial.path}`, function(complete) {
            return me.renderPartial(partial, function(err, result) {
              var ref1, ref2, ref3;
              if (partial.err == null) {
                partial.err = err;
              }
              partial.result = (ref1 = (ref2 = (ref3 = partial.err) != null ? ref3.toString() : void 0) != null ? ref2 : result) != null ? ref1 : '???';
              return complete(partial.err);
            });
          });
          // Return the container
          return partial.container;
        };
        return this;
      }

      // Render the Document
      // Render our partials
      renderDocument(opts, next) {
        var file, filePath, me, partialContainerRegex, partialContainers, tasks, templateData;
        // Prepare
        ({templateData, file} = opts);
        // Check
        partialContainerRegex = /\[partial:([^\]]+)\]/g;
        partialContainers = (opts.content || '').match(partialContainerRegex) || [];
        if (partialContainers.length === 0) {
          return next();
        }
        filePath = file.getFilePath();
        // Prepare
        me = this;
        tasks = new TaskGroup(`Partials for ${filePath}`, {
          concurrency: 0
        }).done(function(err) {
          // Replace containers with results
          opts.content = opts.content.replace(partialContainerRegex, function(match, partialId) {
            var partial;
            // Fetch partial
            partial = me.foundPartials[partialId];
            // Return result
            return partial.result;
          });
          // Complete
          return next(err);
        });
        // Wait for found partials to complete rendering
        partialContainers.forEach(function(partialContainer) {
          var partial, partialId;
          // Fetch partial
          partialId = partialContainer.replace(partialContainerRegex, '$1');
          partial = me.foundPartials[partialId];
          if (partial.task) {
            // Wait for all the partials to complete rendering
            return tasks.addTask(partial.task);
          }
        });
        // Run the tasks
        tasks.run();
        return this;
      }

      // Generate Before
      // Reset the found partials before each generate, otherwise it will get very big
      generateBefore() {
        this.foundPartials = {};
        return this.partialsCache = {};
      }

      // Generate After
      // Reset the found partials after each generate, otherwise it will linger uncessarily
      generateAfter() {
        this.foundPartials = {};
        return this.partialsCache = {};
      }

    };

    // Plugin Name
    PartialsPlugin.prototype.name = 'partials';

    // Default Configuration
    PartialsPlugin.prototype.config = {
      partialPaths: ['partials'],
      collectionName: 'partials',
      performanceFirst: false
    };

    // Locale
    PartialsPlugin.prototype.locale = {
      addingPartial: "Adding partial: %s",
      partialNotFound: "The partial \"%s\" was not found, as such it will not be rendered.",
      renderPartial: "Rendering partial: %s",
      renderedPartial: "Rendered partial: %s",
      renderPartialFailed: "Rendering partial failed: %s. The error follows:"
    };

    // Partial helpers
    PartialsPlugin.prototype.foundPartials = null; // Object

    PartialsPlugin.prototype.partialsCache = null; // Object

    return PartialsPlugin;

  }).call(this);
};
